package de.uni_marburg.powersort.benchmark;

import de.uni_marburg.powersort.data.DataEnum;
import de.uni_marburg.powersort.data.ObjectSupplier;
import de.uni_marburg.powersort.sort.SortEnum;
import de.uni_marburg.powersort.sort.SortImpl;
import org.openjdk.jmh.annotations.BenchmarkMode;
import org.openjdk.jmh.annotations.Fork;
import org.openjdk.jmh.annotations.Level;
import org.openjdk.jmh.annotations.Measurement;
import org.openjdk.jmh.annotations.Mode;
import org.openjdk.jmh.annotations.OutputTimeUnit;
import org.openjdk.jmh.annotations.Scope;
import org.openjdk.jmh.annotations.Setup;
import org.openjdk.jmh.annotations.State;
import org.openjdk.jmh.annotations.Threads;
import org.openjdk.jmh.annotations.Timeout;
import org.openjdk.jmh.annotations.Warmup;

import java.util.concurrent.TimeUnit;

/*
 * Benchmark parameters
 */

@Fork(value = 1, jvmArgsAppend = {"-Xms4g", "-Xmx4g"})
@Threads(1)
@Timeout(time = 10, timeUnit = TimeUnit.SECONDS)
@OutputTimeUnit(TimeUnit.MILLISECONDS)
/*
 * Benchmark mode and related parameters
 */

// AverageTime: "Average time per operation."
// - "This mode is time-based, and it will run until the iteration time expires."
//@BenchmarkMode(Mode.AverageTime)
//@Warmup(iterations = 20, time = 1, timeUnit = TimeUnit.SECONDS)
//@Measurement(iterations = 6, time = 1, timeUnit = TimeUnit.SECONDS)

// SingleShotTime: "Time per single operation"
// - "More warmup/measurement iterations are generally required."
// - "Timers overhead might be significant if benchmarks are small;"
@BenchmarkMode(Mode.SingleShotTime)
// During warmup iterations for ASCENDING_RUNS (with TIM_SORT and FASTER_FINN_SORT):
// - Until the 17th spike of up to +750% (Maybe JVM optimizations happening?)
// - After 40th constant slowdown of around +10% (Maybe CPU frequency adjustments?)
// Thus, we need at least ~50 warmup iterations!
@Warmup(iterations = 60)
@Measurement(iterations = 6)

/*
 * State parameters
 *
 */

// "State objects naturally encapsulate the state on which benchmark is working on."
@State(Scope.Benchmark)

public class JmhBase {
    DataEnum getDataEnum(){
        return null;
    }
    SortEnum getSortEnum(){
        return null;
    }

    private ObjectSupplier data = null;
    Object[] workingCopy;
    SortImpl sortImpl;

    // TODO: This is inaccurate. How to create and use separate arrays for each warmup x iteration x sortAlgorithm ?
    @Setup(Level.Invocation)
    public void setup() {
        sortImpl = getSortEnum().getSortImpl();

        if (Filter.isFiltered(getDataEnum(), getSortEnum())) {
            // This combination of DataEnum and SortEnum should be skipped.
            // We can't tell JMH to not run the benchmark at all.
            // Instead, we let it sort an invalid list which results in an exception.
            // This way the filtered combinations are skipped quickly
            // and not included in the benchmark results.
            data = null;
            workingCopy = null;
            return;
        }

        // A new JmhRunner object is created for each @Param variation.
        // Then, `data` is `null` again.
        if (data == null) {
            data = getDataEnum().getObjectSupplier();
        }
        // For all warmup and measurement iterations of one @Param variation, the JmhRunner object is reused.
        // Thus, we can't just sort `data` directly.
        // Instead, we have to create a copy of it on which the sort algorithm can work.
        // This way, all iterations sort the same input.
        workingCopy = data.getCopy();
    }
}
