package de.uni_marburg.powersort.benchmark;

import de.uni_marburg.powersort.data.ObjectSupplier;
import de.uni_marburg.powersort.sort.SortEnum;

import java.util.concurrent.TimeUnit;

public class CbmBench implements Runnable{
    private final String dataEnumStr;
    private final String sortImplStr;
    private final ObjectSupplier objectSupplier;
    private final SortEnum sortImplementation;

    public CbmBench(String dataEnumStr, String sortImplStr, ObjectSupplier objectSupplier, SortEnum sortImplementation) {
        this.dataEnumStr = dataEnumStr;
        this.sortImplStr = sortImplStr;
        this.objectSupplier = objectSupplier;
        this.sortImplementation = sortImplementation;
    }

    @Override
    public void run() {
        // Warmup
        for (int i = 0; i < 3; i++) {
            benchmark(true);
        }
        // Measure
        for (int i = 0; i < 3; i++) {
            benchmark(false);
        }
    }

    private void benchmark(boolean warmup) {
        System.out.print(dataEnumStr + "," + sortImplStr + ",");
        if (warmup) {
            System.out.print("WARMUP ,");
        } else {
            System.out.print("MEASURE,");
        }
        System.out.flush();

        Object[] sortInput = objectSupplier.getCopy();

        final long startNanos = System.nanoTime();
        sortImplementation.getSortImpl().sort(sortInput);
        final long stopNanos = System.nanoTime();

        final long durNanos = stopNanos - startNanos;
        final long durMillis = TimeUnit.NANOSECONDS.toMillis(durNanos);
        final String durFormatted = LongFormatter.formatUnderscore(durMillis);

        System.out.println(durFormatted);
    }
}
