package de.uni_marburg.powersort.data;

/**
 * Enumeration of Custom Generated Lists (CGL).
 */
public enum CglEnum implements DataEnum {
    RANDOM_INTEGERS,
    ASCENDING_INTEGERS,
    DESCENDING_INTEGERS,
    ASCENDING_RUNS,
    ASCENDING_RUNS_WITH_OVERLAP,
    MANY_ASCENDING_RUNS,
    MANY_ASCENDING_RUNS_WITH_OVERLAP;

    @Override
    public ObjectSupplier getObjectSupplier() {
        // We use a seed to get the same random list every time -> Repeatable benchmarks on same input data!
        // final long seed = 3651660232967549736L; // System.nanoTime() ++ Math.random()
        final long seed = 140506881906827520L; // (long) 'P' * (long) 'O' *(long) 'W' * (long) 'E' * (long) 'R' * (long) 'S' * (long) 'O' * (long) 'R' * (long) 'T';

        int listSize = 66_000_000;
        int runs = 3_010;
        int runLength = 3_010;

        int manyRuns = 120_000;
        int shortRunLength = 50;

        // Constant factors
        double a = 0.96;
        double b = 0.25;
        double c = 0.81;
        double d = 0.85;

        return switch (this) {
            case RANDOM_INTEGERS -> new RandomIntegers(listSize, seed);
            case ASCENDING_INTEGERS -> new AscendingIntegers((int) (a * listSize));
            case DESCENDING_INTEGERS -> new DescendingIntegers((int) (b * listSize));
            case ASCENDING_RUNS -> AscendingRuns.newAscendingRuns(runs, runLength, -1 * runLength);
            case ASCENDING_RUNS_WITH_OVERLAP ->
                    AscendingRuns.newAscendingRuns((int) (c * runs), (int) (c * runLength), (int) (-0.5 * c * runLength));
            case MANY_ASCENDING_RUNS -> AscendingRuns.newAscendingRuns(manyRuns, shortRunLength, -1 * shortRunLength);
            case MANY_ASCENDING_RUNS_WITH_OVERLAP ->
                    AscendingRuns.newAscendingRuns((int) (d * manyRuns), shortRunLength, (int) (-0.5 * shortRunLength));
        };
    }
}
